package fr.unistra.sil.erp.back.repository.stock;

import fr.unistra.sil.erp.back.repository.DatabaseConnectionException;
import fr.unistra.sil.erp.back.repository.DatabaseUpdateException;
import fr.unistra.sil.erp.back.model.Stock;
import fr.unistra.sil.erp.back.repository.IStocksRepository;
import fr.unistra.sil.erp.back.repository.SqliteRepository;
import fr.unistra.sil.erp.back.repository.item.SqliteItemsRepository;
import org.springframework.stereotype.Repository;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author antoine.beauvais
 */
@Repository
public class SqliteStocksRepository extends SqliteRepository implements IStocksRepository {

    public static String columns() {
        List<String> columns = List.of(
                "stocks.id AS stock_id",
                "stocks.item AS stock_item",
                "stocks.quantity AS stock_quantity"
        );
        return String.join(", ", columns) + ", " + SqliteItemsRepository.columns();
    }

    public static Stock fromDb(ResultSet rs) throws SQLException {
        return new Stock(rs.getInt("stock_id"), SqliteItemsRepository.fromDb(rs), rs.getInt("stock_quantity"));
    }

    /**
     * Query used to fetch all stocks stored in the database.
     */
    private static final String SQL_GETSTOCKS =
            "SELECT " + columns() + " FROM stocks INNER JOIN items ON stocks.item = items.id INNER JOIN categories on items.category = categories.id";

    private static final String SQL_GETSTOCKFORITEM = SQL_GETSTOCKS + " WHERE item = ?";

    private static final String SQL_UPDATESTOCK = "UPDATE stocks SET quantity = ? WHERE item = ?";

    /**
     * Class constructor.
     *
     * @param conn
     */
    public SqliteStocksRepository(Connection conn) {
        super(conn);
    }

    /**
     * Retrieves all stock entries from the database.
     *
     * @return the list of stock entries.
     */
    @Override
    public List<Stock> getStocks() {
        ResultSet rs = this.query(SQL_GETSTOCKS);
        if (rs == null)
            return null;

        List<Stock> res = new ArrayList<>();
        try {
            while (rs.next()) {
                Stock s = fromDb(rs);
                res.add(s);
            }
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch results.", ex);
            return null;
        }

        return res;
    }

    /**
     * Returns the stock entry for the specified item.
     *
     * @param itemId the item's ID.
     * @return the Stock entry.
     */
    @Override
    public Stock getStockForItem(int itemId) {
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(SQL_GETSTOCKFORITEM);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            return null;
        }

        try {
            ps.setInt(1, itemId);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            return null;
        }

        ResultSet rs;
        try {
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch stock for item " +
                            itemId, ex);
            return null;
        }

        Stock s;
        try {
            if (rs.next())
                s = fromDb(rs);
            else
                return null;
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to parse results.", ex);
            return null;
        }

        return s;
    }

    /**
     * Updates the specified stock entry with the new quantity.
     *
     * @param stock
     * @throws fr.unistra.sil.erp.back.repository.DatabaseConnectionException
     * @throws fr.unistra.sil.erp.back.repository.DatabaseUpdateException
     */
    @Override
    public void updateStock(Stock stock) throws DatabaseConnectionException, DatabaseUpdateException {
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(SQL_UPDATESTOCK);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            throw new DatabaseConnectionException("Failed to connect to DB.");
        }

        try {
            ps.setInt(1, stock.getQuantity());
            ps.setInt(2, stock.getId());
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            throw new DatabaseUpdateException("Failed to set query params.");
        }

        try {
            ps.execute();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName())
                    .log(Level.SEVERE, "Failed to execute query.", ex);
            throw new DatabaseUpdateException("Failed to execute query.");
        }
    }
}
